# Autogenerated by Astropy's setup.py on 2018-08-02 18:56:18 UTC
from __future__ import unicode_literals
import datetime

version = "3.0.4"
githash = "b24b87fc847e568c7389350d2d1deb7ca5ed9455"


major = 3
minor = 0
bugfix = 4

release = True
timestamp = datetime.datetime(2018, 8, 2, 18, 56, 18)
debug = False

astropy_helpers_version = "3.0.2"

try:
    from ._compiler import compiler
except ImportError:
    compiler = "unknown"

try:
    from .cython_version import cython_version
except ImportError:
    cython_version = "unknown"
